from enum import Enum
from typing import Any, Literal

from pydantic import BaseModel
from typing_extensions import Self


class Operator(Enum):
    EQ = "eq"
    NE = "ne"
    GT = "gt"
    GE = "ge"
    LT = "lt"
    LE = "le"


class ExpressionType(Enum):
    COMPARISON = "comparison"
    # TODO: logic(any, all), and maybe more..


class LogicExpression(BaseModel):
    type: Literal[ExpressionType.COMPARISON] = ExpressionType.COMPARISON
    key: str
    operator: Operator
    value: str | int | float | bool | None

    @classmethod
    def b(cls, key: str, operator: Operator, value: str | int | float | bool) -> Self:
        return cls(key=key, operator=operator, value=value)

    def evaluate(self, data: dict[str, Any]) -> bool:
        value_to_compare = data.get(self.key)
        try:
            match self.operator:
                case Operator.EQ:
                    return value_to_compare == self.value  # type: ignore[operator]
                case Operator.NE:
                    return value_to_compare != self.value  # type: ignore[operator]
                case Operator.GT:
                    return value_to_compare > self.value  # type: ignore[operator]
                case Operator.GE:
                    return value_to_compare >= self.value  # type: ignore[operator]
                case Operator.LT:
                    return value_to_compare < self.value  # type: ignore[operator]
                case Operator.LE:
                    return value_to_compare <= self.value  # type: ignore[operator]
        except TypeError:
            return False

        return False



from typing import Any

from pydantic import BaseModel

from pydantic_logic.expression import LogicExpression


class Logic(BaseModel):
    expressions: list[LogicExpression]

    def evaluate(self, data: dict[str, Any], *, default_if_empty: bool = False) -> bool:
        if len(self.expressions) == 0:
            return default_if_empty
        return all(expr.evaluate(data) for expr in self.expressions)

    def get_wrong_expressions(self, data: dict[str, Any]) -> list[LogicExpression]:
        return [expr for expr in self.expressions if not expr.evaluate(data)]
